@@ -229,9 +229,9 @@ grub_hfs_block (struct grub_hfs_data *data, grub_hfs_datarecord_t dat,
    POS.  Return the amount of read bytes in READ.  */
 static grub_ssize_t
 grub_hfs_read_file (struct grub_hfs_data *data,
-		    void (*read_hook) (unsigned long sector,
+		    void (*read_hook) (grub_disk_addr_t sector,
 				       unsigned offset, unsigned length),
-		     int pos, unsigned int len, char *buf)
+		     int pos, grub_size_t len, char *buf)
 {
   int i;
   int blockcnt;
@@ -261,7 +261,7 @@ grub_hfs_read_file (struct grub_hfs_data *data,
 	  blockend = (len + pos) % data->blksz;
 	  
 	  /* The last portion is exactly EXT2_BLOCK_SIZE (data).  */
-	  if (!blockend)
+	  if (! blockend)
 	    blockend = data->blksz;
 	}
 
@@ -804,7 +804,7 @@ grub_hfs_open (struct grub_file *file, const char *name)
 }
 
 static grub_ssize_t
-grub_hfs_read (grub_file_t file, char *buf, grub_ssize_t len)
+grub_hfs_read (grub_file_t file, char *buf, grub_size_t len)
 {
   struct grub_hfs_data *data = 
     (struct grub_hfs_data *) file->data;
