@@ -1,20 +1,21 @@
 /* fs.h - filesystem manager */
 /*
  *  GRUB  --  GRand Unified Bootloader
- *  Copyright (C) 2002,2003,2004,2007,2008  Free Software Foundation, Inc.
+ *  Copyright (C) 2002,2003,2004  Free Software Foundation, Inc.
  *
- *  GRUB is free software: you can redistribute it and/or modify
+ *  GRUB is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation, either version 3 of the License, or
+ *  the Free Software Foundation; either version 2 of the License, or
  *  (at your option) any later version.
  *
- *  GRUB is distributed in the hope that it will be useful,
+ *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with GRUB.  If not, see <http://www.gnu.org/licenses/>.
+ *  along with GRUB; if not, write to the Free Software
+ *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
 #ifndef GRUB_FS_HEADER
@@ -27,14 +28,6 @@
 /* Forward declaration is required, because of mutual reference.  */
 struct grub_file;
 
-struct grub_dirhook_info
-{
-  int dir:1;
-  int mtimeset:1;
-  int case_insensitive:1;
-  grub_int32_t mtime;
-};
-
 /* Filesystem descriptor.  */
 struct grub_fs
 {
@@ -43,36 +36,22 @@ struct grub_fs
 
   /* Call HOOK with each file under DIR.  */
   grub_err_t (*dir) (grub_device_t device, const char *path,
-		     int (*hook) (const char *filename,
-				  const struct grub_dirhook_info *info));
-
+		     int (*hook) (const char *filename, int dir));
+  
   /* Open a file named NAME and initialize FILE.  */
   grub_err_t (*open) (struct grub_file *file, const char *name);
-
+  
   /* Read LEN bytes data from FILE into BUF.  */
   grub_ssize_t (*read) (struct grub_file *file, char *buf, grub_size_t len);
-
+  
   /* Close the file FILE.  */
   grub_err_t (*close) (struct grub_file *file);
-
+  
   /* Return the label of the device DEVICE in LABEL.  The label is
      returned in a grub_malloc'ed buffer and should be freed by the
      caller.  */
   grub_err_t (*label) (grub_device_t device, char **label);
 
-  /* Return the uuid of the device DEVICE in UUID.  The uuid is
-     returned in a grub_malloc'ed buffer and should be freed by the
-     caller.  */
-  grub_err_t (*uuid) (grub_device_t device, char **uuid);
-
-  /* Get writing time of filesystem. */
-  grub_err_t (*mtime) (grub_device_t device, grub_int32_t *timebuf);
-
-#ifdef GRUB_UTIL
-  /* Whether this filesystem reserves first sector for DOS-style boot.  */
-  int reserved_first_sector;
-#endif
-
   /* The next filesystem.  */
   struct grub_fs *next;
 };
@@ -93,4 +72,26 @@ void EXPORT_FUNC(grub_fs_unregister) (grub_fs_t fs);
 void EXPORT_FUNC(grub_fs_iterate) (int (*hook) (const grub_fs_t fs));
 grub_fs_t EXPORT_FUNC(grub_fs_probe) (grub_device_t device);
 
+#ifdef GRUB_UTIL
+void grub_fat_init (void);
+void grub_fat_fini (void);
+void grub_ext2_init (void);
+void grub_ext2_fini (void);
+void grub_ufs_init (void);
+void grub_ufs_fini (void);
+void grub_minix_init (void);
+void grub_minix_fini (void);
+void grub_hfs_init (void);
+void grub_hfs_fini (void);
+void grub_jfs_init (void);
+void grub_jfs_fini (void);
+void grub_xfs_init (void);
+void grub_xfs_fini (void);
+void grub_affs_init (void);
+void grub_affs_fini (void);
+void grub_sfs_init (void);
+void grub_sfs_fini (void);
+void grub_iso9660_init (void);
+#endif /* GRUB_UTIL */
+
 #endif /* ! GRUB_FS_HEADER */
