@@ -22,10 +22,9 @@
 
 #include <grub/symbol.h>
 
+#define GRUB_IEEE1275_MOD_ALIGN 0x1000
+
 void EXPORT_FUNC (grub_reboot) (void);
 void EXPORT_FUNC (grub_halt) (void);
 
-/* Where grub-mkimage places the core modules in memory.  */
-#define GRUB_IEEE1275_MODULE_BASE 0x00300000
-
 #endif /* ! GRUB_KERNEL_MACHINE_HEADER */
