@@ -1,6 +1,6 @@
 /*
  *  GRUB  --  GRand Unified Bootloader
- *  Copyright (C) 2003  Free Software Foundation, Inc.
+ *  Copyright (C) 2003,2006  Free Software Foundation, Inc.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -26,11 +26,9 @@ typedef jmp_buf grub_jmp_buf;
 #define grub_setjmp setjmp
 #define grub_longjmp longjmp
 #else
-/* This must define grub_jmp_buf.  */
-#include <grub/cpu/setjmp.h>
-
-int grub_setjmp (grub_jmp_buf env);
-void grub_longjmp (grub_jmp_buf env, int val) __attribute__ ((noreturn));
+/* This must define grub_jmp_buf, and declare grub_setjmp and
+   grub_longjmp.  */
+# include <grub/cpu/setjmp.h>
 #endif
 
 #endif /* ! GRUB_SETJMP_HEADER */
