@@ -23,6 +23,8 @@
 #include <config.h>
 #include <grub/cpu/types.h>
 
+#define UNUSED __attribute__ ((unused))
+
 #ifdef GRUB_UTIL
 # define GRUB_CPU_SIZEOF_VOID_P	SIZEOF_VOID_P
 # define GRUB_CPU_SIZEOF_LONG	SIZEOF_LONG
@@ -69,32 +71,33 @@ typedef unsigned long long	grub_uint64_t;
 #endif
 
 /* Misc types.  */
-#if SIZEOF_VOID_P == 8
-typedef grub_uint64_t	grub_host_addr_t;
-typedef grub_uint64_t	grub_host_off_t;
-typedef grub_uint64_t	grub_host_size_t;
-typedef grub_int64_t	grub_host_ssize_t;
+#if GRUB_TARGET_SIZEOF_VOID_P == 8
+typedef grub_uint64_t	grub_target_addr_t;
+typedef grub_uint64_t	grub_target_off_t;
+typedef grub_uint64_t	grub_target_size_t;
+typedef grub_int64_t	grub_target_ssize_t;
 #else
-typedef grub_uint32_t	grub_host_addr_t;
-typedef grub_uint32_t	grub_host_off_t;
-typedef grub_uint32_t	grub_host_size_t;
-typedef grub_int32_t	grub_host_ssize_t;
+typedef grub_uint32_t	grub_target_addr_t;
+typedef grub_uint32_t	grub_target_off_t;
+typedef grub_uint32_t	grub_target_size_t;
+typedef grub_int32_t	grub_target_ssize_t;
 #endif
 
 #if GRUB_CPU_SIZEOF_VOID_P == 8
 typedef grub_uint64_t	grub_addr_t;
-typedef grub_uint64_t	grub_off_t;
 typedef grub_uint64_t	grub_size_t;
 typedef grub_int64_t	grub_ssize_t;
 #else
 typedef grub_uint32_t	grub_addr_t;
-typedef grub_uint32_t	grub_off_t;
 typedef grub_uint32_t	grub_size_t;
 typedef grub_int32_t	grub_ssize_t;
 #endif
 
-/* FIXME: Will be grub_uint64_t */
-typedef unsigned long	grub_disk_addr_t;
+/* The type for representing a file offset.  */
+typedef grub_uint64_t	grub_off_t;
+
+/* The type for representing a disk block address.  */
+typedef grub_uint64_t	grub_disk_addr_t;
 
 /* Byte-orders.  */
 #define grub_swap_bytes16(x)	\
