@@ -1,24 +1,24 @@
 /* of.c - Access the Open Firmware client interface.  */
 /*
  *  GRUB  --  GRand Unified Bootloader
- *  Copyright (C) 2003,2004,2005,2007,2008  Free Software Foundation, Inc.
+ *  Copyright (C) 2003, 2004, 2005  Free Software Foundation, Inc.
  *
- *  GRUB is free software: you can redistribute it and/or modify
+ *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation, either version 3 of the License, or
+ *  the Free Software Foundation; either version 2 of the License, or
  *  (at your option) any later version.
  *
- *  GRUB is distributed in the hope that it will be useful,
+ *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with GRUB.  If not, see <http://www.gnu.org/licenses/>.
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
 #include <grub/ieee1275/ieee1275.h>
-#include <grub/types.h>
 
 #define IEEE1275_PHANDLE_INVALID  ((grub_ieee1275_phandle_t) -1)
 #define IEEE1275_IHANDLE_INVALID  ((grub_ieee1275_ihandle_t) 0)
@@ -26,6 +26,16 @@
 
 
 
+/* FIXME is this function needed? */
+grub_uint32_t
+grub_ieee1275_decode_int_4 (unsigned char *p)
+{
+  grub_uint32_t val = (*p++ << 8);
+  val = (val + *p++) << 8;
+  val = (val + *p++) << 8;
+  return (val + *p);
+}
+
 int
 grub_ieee1275_finddevice (char *name, grub_ieee1275_phandle_t *phandlep)
 {
@@ -79,26 +89,6 @@ grub_ieee1275_get_property (grub_ieee1275_phandle_t phandle,
   return 0;
 }
 
-int
-grub_ieee1275_get_integer_property (grub_ieee1275_phandle_t phandle,
-				    const char *property, grub_uint32_t *buf,
-				    grub_size_t size, grub_ssize_t *actual)
-{
-  int ret;
-  ret = grub_ieee1275_get_property (phandle, property, (void *) buf, size, actual);
-#ifndef GRUB_CPU_WORDS_BIGENDIAN
-  /* Integer properties are always in big endian.  */
-  if (ret == 0)
-    {
-      int i;
-      size /= sizeof (grub_uint32_t);
-      for (i = 0; i < size; i++)
-	buf[i] = grub_be_to_cpu32 (buf[i]);
-    }
-#endif
-  return ret;
-}
-
 int
 grub_ieee1275_next_property (grub_ieee1275_phandle_t phandle, char *prev_prop,
 			     char *prop)
