@@ -38,12 +38,10 @@ grub_children_iterate (char *devpath,
   grub_ieee1275_phandle_t dev;
   grub_ieee1275_phandle_t child;
 
-  grub_ieee1275_finddevice (devpath, &dev);
-  if (dev == (grub_ieee1275_phandle_t) -1)
+  if (grub_ieee1275_finddevice (devpath, &dev))
     return grub_error (GRUB_ERR_UNKNOWN_DEVICE, "Unknown device");
 
-  grub_ieee1275_child (dev, &child);
-  if (child == (grub_ieee1275_phandle_t) -1)
+  if (grub_ieee1275_child (dev, &child))
     return grub_error (GRUB_ERR_BAD_DEVICE, "Device has no children");
 
   do
@@ -56,19 +54,16 @@ grub_children_iterate (char *devpath,
       struct grub_ieee1275_devalias alias;
       int actual;
 
-      grub_ieee1275_get_property (child, "device_type", &childtype,
-				  sizeof childtype, &actual);
-      if (actual == -1)
+      if (grub_ieee1275_get_property (child, "device_type", &childtype,
+				      sizeof childtype, &actual))
 	continue;
 
-      grub_ieee1275_package_to_path (child, childpath, sizeof childpath,
-      				     &actual);
-      if (actual == -1)
+      if (grub_ieee1275_package_to_path (child, childpath, sizeof childpath,
+					 &actual))
 	continue;
 
-      grub_ieee1275_get_property (child, "name", &childname,
-				  sizeof childname, &actual);
-      if (actual == -1)
+      if (grub_ieee1275_get_property (child, "name", &childname,
+				      sizeof childname, &actual))
 	continue;
 
       grub_sprintf (fullname, "%s/%s", devpath, childname);
@@ -88,63 +83,52 @@ grub_children_iterate (char *devpath,
 grub_err_t
 grub_devalias_iterate (int (*hook) (struct grub_ieee1275_devalias *alias))
 {
-  grub_ieee1275_phandle_t devalias;
+  grub_ieee1275_phandle_t aliases;
   char aliasname[32];
   int actual;
   struct grub_ieee1275_devalias alias;
 
-  if (grub_ieee1275_finddevice ("/aliases", &devalias))
+  if (grub_ieee1275_finddevice ("/aliases", &aliases))
     return -1;
 
-  /* XXX: Is this the right way to find the first property?  */
+  /* Find the first property.  */
   aliasname[0] = '\0';
 
-  /* XXX: Are the while conditions correct?  */
-  while (grub_ieee1275_next_property (devalias, aliasname, aliasname, &actual)
-	 || actual)
+  while (grub_ieee1275_next_property (aliases, aliasname, aliasname))
     {
       grub_ieee1275_phandle_t dev;
-      grub_size_t pathlen;
+      grub_ssize_t pathlen;
       char *devpath;
       /* XXX: This should be large enough for any possible case.  */
       char devtype[64];
-  
-      grub_ieee1275_get_property_length (devalias, aliasname, &pathlen);
+
+      grub_ieee1275_get_property_length (aliases, aliasname, &pathlen);
 
       /* The property `name' is a special case we should skip.  */
       if (!grub_strcmp (aliasname, "name"))
-	  continue;
-      
+	continue;
+
       devpath = grub_malloc (pathlen);
       if (! devpath)
 	return grub_errno;
 
-      if (grub_ieee1275_get_property (devalias, aliasname, devpath, pathlen,
+      if (grub_ieee1275_get_property (aliases, aliasname, devpath, pathlen,
 				      &actual))
-	{
-	  grub_free (devpath);
-	  continue;
-	}
-      
-      if (grub_ieee1275_finddevice (devpath, &dev)
-	  || dev == (grub_ieee1275_phandle_t) -1)
-	{
-	  grub_free (devpath);
-	  continue;
-	}
+	goto nextprop;
 
-      if (grub_ieee1275_get_property (dev, "device_type", devtype, sizeof devtype,
-				      &actual))
-	{
-	  grub_free (devpath);
-	  continue;
-	}
+      if (grub_ieee1275_finddevice (devpath, &dev))
+	goto nextprop;
+
+      if (grub_ieee1275_get_property (dev, "device_type", devtype,
+				      sizeof devtype, &actual))
+	goto nextprop;
 
       alias.name = aliasname;
-      alias.path= devpath;
+      alias.path = devpath;
       alias.type = devtype;
       hook (&alias);
-      
+
+nextprop:
       grub_free (devpath);
     }
 
