@@ -18,7 +18,6 @@
  */
 
 #include <stdlib.h>
-#include <malloc.h>
 #include <sys/stat.h>
 #include <argp.h>
 #include <string.h>
@@ -186,7 +185,7 @@ main (int argc, char *argv[])
   /* Make sure that there is a root device.  */
   if (! args.root_dev)
     {
-      args.root_dev = grub_guess_root_device (args.dir ? : DEFAULT_DIRECTORY);
+      args.root_dev = grub_util_biosdisk_get_grub_dev (grub_guess_root_device (args.dir ? : DEFAULT_DIRECTORY));
       if (! args.root_dev)
 	{
 	  grub_util_info ("guessing the root device failed, because of `%s'",
