@@ -32,6 +32,8 @@
 #include <grub/machine/boot.h>
 #include <grub/machine/kernel.h>
 #include <grub/term.h>
+#include <grub/util/raid.h>
+#include <grub/util/lvm.h>
 
 #include <stdio.h>
 #include <unistd.h>
@@ -60,7 +62,7 @@
 /* This is the blocklist used in the diskboot image.  */
 struct boot_blocklist
 {
-  grub_uint32_t start;
+  grub_uint64_t start;
   grub_uint16_t len;
   grub_uint16_t segment;
 } __attribute__ ((packed));
@@ -91,7 +93,7 @@ grub_refresh (void)
 static void
 setup (const char *prefix, const char *dir,
        const char *boot_file, const char *core_file,
-       const char *root, const char *dest)
+       const char *root, const char *dest, int must_embed)
 {
   char *boot_path, *core_path;
   char *boot_img, *core_img;
@@ -99,14 +101,14 @@ setup (const char *prefix, const char *dir,
   grub_uint16_t core_sectors;
   grub_device_t root_dev, dest_dev;
   grub_uint8_t *boot_drive;
-  grub_uint32_t *kernel_sector;
+  grub_disk_addr_t *kernel_sector;
   grub_uint16_t *boot_drive_check;
   struct boot_blocklist *first_block, *block;
   grub_int32_t *install_dos_part, *install_bsd_part;
   char *install_prefix;
   char *tmp_img;
   int i;
-  unsigned long first_sector;
+  grub_disk_addr_t first_sector;
   grub_uint16_t current_segment
     = GRUB_BOOT_MACHINE_KERNEL_SEG + (GRUB_DISK_SECTOR_SIZE >> 4);
   grub_uint16_t last_length = GRUB_DISK_SECTOR_SIZE;
@@ -114,9 +116,9 @@ setup (const char *prefix, const char *dir,
   FILE *fp;
   unsigned long first_start = ~0UL;
   
-  auto void save_first_sector (unsigned long sector, unsigned offset,
+  auto void save_first_sector (grub_disk_addr_t sector, unsigned offset,
 			       unsigned length);
-  auto void save_blocklists (unsigned long sector, unsigned offset,
+  auto void save_blocklists (grub_disk_addr_t sector, unsigned offset,
 			     unsigned length);
 
   auto int find_first_partition_start (grub_disk_t disk,
@@ -135,10 +137,10 @@ setup (const char *prefix, const char *dir,
       return 0;
     }
   
-  void save_first_sector (unsigned long sector, unsigned offset,
+  void save_first_sector (grub_disk_addr_t sector, unsigned offset,
 			  unsigned length)
     {
-      grub_util_info ("the fist sector is <%lu,%u,%u>",
+      grub_util_info ("the first sector is <%llu,%u,%u>",
 		      sector, offset, length);
       
       if (offset != 0 || length != GRUB_DISK_SECTOR_SIZE)
@@ -147,23 +149,24 @@ setup (const char *prefix, const char *dir,
       first_sector = sector;
     }
 
-  void save_blocklists (unsigned long sector, unsigned offset, unsigned length)
+  void save_blocklists (grub_disk_addr_t sector, unsigned offset,
+			unsigned length)
     {
       struct boot_blocklist *prev = block + 1;
 
-      grub_util_info ("saving <%lu,%u,%u> with the segment 0x%x",
+      grub_util_info ("saving <%llu,%u,%u> with the segment 0x%x",
 		      sector, offset, length, (unsigned) current_segment);
       
       if (offset != 0 || last_length != GRUB_DISK_SECTOR_SIZE)
 	grub_util_error ("Non-sector-aligned data is found in the core file");
 
       if (block != first_block
-	  && (grub_le_to_cpu32 (prev->start)
+	  && (grub_le_to_cpu64 (prev->start)
 	      + grub_le_to_cpu16 (prev->len)) == sector)
 	prev->len = grub_cpu_to_le16 (grub_le_to_cpu16 (prev->len) + 1);
       else
 	{
-	  block->start = grub_cpu_to_le32 (sector);
+	  block->start = grub_cpu_to_le64 (sector);
 	  block->len = grub_cpu_to_le16 (1);
 	  block->segment = grub_cpu_to_le16 (current_segment);
 
@@ -187,7 +190,7 @@ setup (const char *prefix, const char *dir,
 
   /* Set the addresses of BOOT_DRIVE, KERNEL_SECTOR and BOOT_DRIVE_CHECK.  */
   boot_drive = (grub_uint8_t *) (boot_img + GRUB_BOOT_MACHINE_BOOT_DRIVE);
-  kernel_sector = (grub_uint32_t *) (boot_img
+  kernel_sector = (grub_disk_addr_t *) (boot_img
 				     + GRUB_BOOT_MACHINE_KERNEL_SECTOR);
   boot_drive_check = (grub_uint16_t *) (boot_img
 					+ GRUB_BOOT_MACHINE_DRIVE_CHECK);
@@ -217,9 +220,12 @@ setup (const char *prefix, const char *dir,
 		    + GRUB_KERNEL_MACHINE_PREFIX);
 
   /* Open the root device and the destination device.  */
-  root_dev = grub_device_open (root);
-  if (! root_dev)
-    grub_util_error ("%s", grub_errmsg);
+  if (!must_embed)
+    {
+      root_dev = grub_device_open (root);
+      if (! root_dev)
+	grub_util_error ("%s", grub_errmsg);
+    }
 
   dest_dev = grub_device_open (dest);
   if (! dest_dev)
@@ -266,7 +272,7 @@ setup (const char *prefix, const char *dir,
 	  grub_util_info ("will embed the core image into after the MBR");
 	  
 	  /* The first blocklist contains the whole sectors.  */
-	  first_block->start = grub_cpu_to_le32 (2);
+	  first_block->start = grub_cpu_to_le64 (2);
 	  first_block->len = grub_cpu_to_le16 (core_sectors - 1);
 	  first_block->segment
 	    = grub_cpu_to_le16 (GRUB_BOOT_MACHINE_KERNEL_SEG
@@ -279,7 +285,9 @@ setup (const char *prefix, const char *dir,
 	  block->segment = 0;
 
 	  /* Embed information about the installed location.  */
-	  if (root_dev->disk->partition)
+	  if (must_embed)
+	    *install_dos_part = *install_bsd_part = grub_cpu_to_le32 (-2);
+	  else if (root_dev->disk->partition)
 	    {
 	      struct grub_pc_partition *pcdata =
 		root_dev->disk->partition->data;
@@ -305,7 +313,7 @@ setup (const char *prefix, const char *dir,
 	  /* The boot image and the core image are on the same drive,
 	     so there is no need to specify the boot drive explicitly.  */
 	  *boot_drive = 0xff;
-	  *kernel_sector = grub_cpu_to_le32 (1);
+	  *kernel_sector = grub_cpu_to_le64 (1);
 
 	  /* Write the boot image onto the disk.  */
 	  if (grub_disk_write (dest_dev->disk, 0, 0, GRUB_DISK_SECTOR_SIZE,
@@ -315,6 +323,9 @@ setup (const char *prefix, const char *dir,
 	  goto finish;
 	}
     }
+  else if (must_embed)
+    grub_util_error ("Can't embed the core image, but this is required when\n"
+		     "the root device is on a RAID array or LVM volume.");
   
   /* The core image must be put on a filesystem unfortunately.  */
   grub_util_info ("will leave the core image on the filesystem");
@@ -339,7 +350,7 @@ setup (const char *prefix, const char *dir,
       file = grub_file_open (core_path);
       if (file)
 	{
-	  if (grub_file_size (file) != (grub_ssize_t) core_size)
+	  if (grub_file_size (file) != core_size)
 	    grub_util_info ("succeeded in opening the core image but the size is different (%d != %d)",
 			    (int) grub_file_size (file), (int) core_size);
 	  else if (grub_file_read (file, tmp_img, core_size)
@@ -421,10 +432,12 @@ setup (const char *prefix, const char *dir,
       != (grub_ssize_t) core_size - GRUB_DISK_SECTOR_SIZE)
     grub_util_error ("Failed to read the rest sectors of the core image");
 
+  grub_file_close (file);
+  
   free (core_path);
   free (tmp_img);
   
-  *kernel_sector = grub_cpu_to_le32 (first_sector);
+  *kernel_sector = grub_cpu_to_le64 (first_sector);
 
   /* If the destination device is different from the root device,
      it is necessary to embed the boot drive explicitly.  */
@@ -480,7 +493,8 @@ setup (const char *prefix, const char *dir,
   free (core_img);
   free (boot_img);
   grub_device_close (dest_dev);
-  grub_device_close (root_dev);
+  if (!must_embed)
+    grub_device_close (root_dev);
 }
 
 static struct option options[] =
@@ -547,6 +561,7 @@ main (int argc, char *argv[])
   char *root_dev = 0;
   char *prefix;
   char *dest_dev;
+  int must_embed = 0;
   
   progname = "grub-setup";
 
@@ -668,7 +683,7 @@ main (int argc, char *argv[])
     }
   else
     {
-      root_dev = grub_guess_root_device (dir ? : DEFAULT_DIRECTORY);
+      root_dev = grub_util_biosdisk_get_grub_dev (grub_guess_root_device (dir ? : DEFAULT_DIRECTORY));
       if (! root_dev)
 	{
 	  grub_util_info ("guessing the root device failed, because of `%s'",
@@ -677,12 +692,50 @@ main (int argc, char *argv[])
 	}
     }
 
+#ifdef __linux__
+  if (grub_util_lvm_isvolume (root_dev))
+    {
+      char *newprefix;
+      must_embed = 1;
+
+      newprefix = xmalloc (1 + strlen (root_dev) + 1 + strlen (prefix) + 1);
+      sprintf (newprefix, "(%s)%s", root_dev, prefix);
+      free (prefix);
+      prefix = newprefix;
+    }
+    
+  if (dest_dev[0] == 'm' && dest_dev[1] == 'd'
+      && dest_dev[2] >= '0' && dest_dev[2] <= '9')
+    {
+      char **devicelist;
+      char *raid_prefix;
+      int i;
+
+      raid_prefix = xmalloc (1 + strlen (dest_dev) + 1 + strlen (prefix) + 1);
+
+      sprintf (raid_prefix, "(%s)%s", dest_dev, prefix);
+      
+      devicelist = grub_util_raid_getmembers (dest_dev);
+
+      for (i = 0; devicelist[i]; i++)
+	{
+	  setup (raid_prefix,
+		 dir ? : DEFAULT_DIRECTORY,
+		 boot_file ? : DEFAULT_BOOT_FILE,
+		 core_file ? : DEFAULT_CORE_FILE,
+		 root_dev, grub_util_biosdisk_get_grub_dev (devicelist[i]), 1);
+	}
+
+      free (raid_prefix);
+    }
+  else
+#endif
   /* Do the real work.  */
-  setup (prefix,
-	 dir ? : DEFAULT_DIRECTORY,
-	 boot_file ? : DEFAULT_BOOT_FILE,
-	 core_file ? : DEFAULT_CORE_FILE,
-	 root_dev, dest_dev);
+    setup (prefix,
+	   dir ? : DEFAULT_DIRECTORY,
+	   boot_file ? : DEFAULT_BOOT_FILE,
+	   core_file ? : DEFAULT_CORE_FILE,
+	   root_dev, dest_dev, must_embed);
 
   /* Free resources.  */
   grub_ext2_fini ();
